/*
 * GQoob
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqoob.h"
#include "io_xine.h"

#include "display.h"
#include "display_menu.h"
#include "display_xine.h"
#include "io_mpg123.h"
#include "io_ogg123.h"
#include "menus.h"
#include "players.h"
#include "playlist.h"
#include "songinfo.h"
#include "ui_fileops.h"
#include "ui_utildlg.h"
#include "utils_xine.h"
#include "window.h"

#include <gdk/gdkx.h>

#if 0
#include <xine.h>

#include <X11/Xlib.h>
#include <xine/video_out_x11.h>
#endif

#include "gtkxine.h"


#define GQOOB_XINE_RC_FILE "xine.config"

#define FULL_SCREEN_HIDE_MOUSE_DELAY 3000

/* config */

gchar *gqoob_xine_output_video = NULL;
gchar *gqoob_xine_output_audio = NULL;

/* XINE stuff */

static GtkWidget *gqoob_xine_vw = NULL;
static GtkWidget *gqoob_xine_fixed = NULL;

static gchar *gqoob_xine_override_video = NULL;
static gchar *gqoob_xine_override_audio = NULL;


/* gqoob stuff */

static gint gqoob_xine_idle_id = -1;
static gint gqoob_xine_x = 0;
static gint gqoob_xine_y = 0;
static gint gqoob_xine_w = 32;
static gint gqoob_xine_h = 32;

gint gqoob_xine_debug = FALSE;

static GList *ext_break_list(const gchar *text)
{
	GList *list = NULL;
	const gchar *b, *e;

	if (!text) return NULL;

	b = text;
	e = b;
	while (*b != '\0')
		{
		gint l = 0;

		while (*e != ' ' && *e != '\0')
			{
			e++;
			l++;
			}
		while (*e == ' ')
			{
			e++;
			}

		list = g_list_prepend(list, g_strndup(b, l));

		b = e;
		}

	return g_list_reverse(list);
}

/*
 *----------------------------------------------------------------------------
 * mime types (weeeeeeeee!)
 * note: the xine util to get a list from available plugins is um crashing badly
 * so instead use predefined set (sorry)
 *----------------------------------------------------------------------------
 */

static gchar *gqoob_xine_types[] = {
	"Mpeg video", ".mpg .mpeg .mpe .m2v .mpv .vob",
	"Mpeg audio", ".mp3 .mp2 .mpa .mpega",
	"OGG Vorbis audio", ".ogg",
	"Wave audio", ".wav",
	"AVI video", ".avi",
	"MS video/audio", ".asf .wma .wmv",
	"Quicktime", ".mov .qt",
	"Motion Jpeg", ".mjpg .smjpeg",
	"Film", ".cak .cpk .film",
	"Animation", ".fli .flc",
	"Misc", ".cin .m2t .roq .ts .trp .vdr",
	NULL, NULL
};

static gint copy_io_module(IO_ModuleData *imd, const gchar *description)
{
	IO_ModuleData *fake_imd;

	fake_imd = g_new0(IO_ModuleData, 1);
	memcpy(fake_imd, imd, sizeof(IO_ModuleData));
	fake_imd->config_load_func = NULL;
	fake_imd->config_save_func = NULL;
	fake_imd->config_init_func = NULL;
	fake_imd->config_apply_func = NULL;
	fake_imd->config_close_func = NULL;
	fake_imd->description = g_strdup(description);

	return player_module_register(fake_imd);
}

static void gqoob_xine_register_types(IO_ModuleData *imd, gint id)
{
	gint i;
	gint original_used = FALSE;

	i = 0;
	while (gqoob_xine_types[i])
		{
		GList *list;
		GList *work;
		gint copy_id;

		if (original_used)
			{
			copy_id = copy_io_module(imd, gqoob_xine_types[i]);
			}
		else
			{
			copy_id = id;
			original_used = TRUE;
			}
		list = ext_break_list(gqoob_xine_types[i + 1]);
		work = list;
		while (work)
			{
			gchar *ext;

			ext = work->data;
			work = work->next;

			module_register_file_suffix_type(ext, gqoob_xine_types[i], copy_id);
			}
		path_list_free(list);
		i += 2;
		}
	
#if 0
	config_values_t *config;
	char *identifiers = NULL;
	char *mimetypes = NULL;
	gchar *filename;

	filename = g_strconcat(homedir(), "/.xine/config", NULL);
	config = xine_config_file_init(filename);
	g_free(filename);

	if (!config) return;

	/* ok, the plugins just plain crash on calling this (even tried on std config in gtkxine.c) */
	xine_list_demux_plugins(config, &identifiers, &mimetypes);

	printf("demuxer: \"%s\" \"%s\"\n", identifiers, mimetypes);

	if (identifiers) free(identifiers);
	if (mimetypes) free(mimetypes);

	/* FIXME: how do we free a config type ? */
#endif
}

/*
 *----------------------------------------------------------------------------
 * plugins (video/audio output)
 *----------------------------------------------------------------------------
 */

static GList *gqoob_xine_output_video_list = NULL;
static GList *gqoob_xine_output_audio_list = NULL;

/* do NOT free returned list */
GList *gqoob_xine_get_video_list(void)
{
	if (!gqoob_xine_output_video_list)
		{
		char **array;
		gint i;

		array = xine_list_video_output_plugins(VISUAL_TYPE_X11);
		i = 0;
		while (array && array[i])
			{
			gqoob_xine_output_video_list = g_list_append(gqoob_xine_output_video_list, g_strdup(array[i]));
			i++;
			}
			
		if (array) free(array);

		gqoob_xine_output_video_list = g_list_prepend(gqoob_xine_output_video_list, g_strdup("auto"));
		}
	
	return gqoob_xine_output_video_list;
}

/* do NOT free returned list */
GList *gqoob_xine_get_audio_list(void)
{
        if (!gqoob_xine_output_audio_list)
		{
		char **array;
		gint i;

		array = xine_list_audio_output_plugins();
		i = 0;
		while (array && array[i])
			{
			gqoob_xine_output_audio_list = g_list_append(gqoob_xine_output_audio_list, g_strdup(array[i]));
			i++;
			}

		if (array) free(array);

		gqoob_xine_output_audio_list = g_list_prepend(gqoob_xine_output_audio_list, g_strdup("auto"));
		gqoob_xine_output_audio_list = g_list_append(gqoob_xine_output_audio_list, g_strdup("null"));
		}
	return gqoob_xine_output_audio_list;	
}

static gint simple_list_find(GList *list, const gchar *needle)
{
	if (!needle) return FALSE;

	while (list)
		{
		const gchar *haystack;

		haystack = list->data;
		list = list->next;

		if (strcmp(needle, haystack) == 0) return TRUE;
		}

	return FALSE;
}

gint gqoob_xine_override_plugin_config(const gchar *video, const gchar *audio)
{
	if (video)
		{
		if (!simple_list_find(gqoob_xine_get_video_list(), video))
			{
			printf(_("Video output plugin \"%s\" not found.\n"), video);
			return FALSE;
			}
		g_free(gqoob_xine_override_video);
		gqoob_xine_override_video = g_strdup(video);
		}
	if (audio)
		{
		if (!simple_list_find(gqoob_xine_get_audio_list(), audio))
			{
			printf(_("Audio output plugin \"%s\" not found.\n"), audio);
			return FALSE;
			}
		g_free(gqoob_xine_override_audio);
		gqoob_xine_override_audio = g_strdup(audio);
		}

	return TRUE;
}

void gqoob_xine_print_plugins(void)
{
	GList *work;

	printf(_("Available video plugins:\n"));
	work = gqoob_xine_get_video_list();
	while (work)
		{
		const gchar *name;

		name = work->data;
		work = work->next;
		printf("  %s\n", name);
		}

	printf(_("Available audio plugins:\n"));
	work = gqoob_xine_get_audio_list();
	while (work)
		{
		const gchar *name;

		name = work->data;
		work = work->next;
		printf("  %s\n", name);
		}
}

/*
 *----------------------------------------------------------------------------
 * full screen
 *----------------------------------------------------------------------------
 */

static gint gqoob_xine_cursor_timer_id = -1;
static GtkWidget *gqoob_xine_cursor_window = NULL;

static gint gqoob_xine_mouse_press_cb(GtkWidget *w, GdkEventButton *event, gpointer data)
{
	if (event && event->button == 3)
		{
		if (skinned_menus_enable)
			{
			display_menu_popup(main_window->skin_path, -1, -1, event->button, event->time, main_window);
			}
		else
			{
			GtkWidget *menu;
			menu = menu_main(main_window);
			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
			}
		}
	else if (event && event->button == 1)
		{
		btn_fullscreen_controls_toggle_pressed();
		}

	return TRUE;
}

static gint gqoob_xine_cursor_cb(gpointer data)
{
	if (gqoob_xine_vw && gtk_xine_is_fullscreen(GTK_XINE(gqoob_xine_vw)))
		{
		if (gtk_xine_get_show_cursor(GTK_XINE(gqoob_xine_vw)))
			{
			gtk_xine_set_show_cursor(GTK_XINE(gqoob_xine_vw), FALSE);
			}
		}

	if (gqoob_xine_cursor_window && GTK_WIDGET_VISIBLE(gqoob_xine_cursor_window))
		{
		gtk_widget_hide(gqoob_xine_cursor_window);
		}
	
	gqoob_xine_cursor_timer_id = -1;
	return FALSE;
}

static void gqoob_xine_cursor_disable(void)
{
	if (gqoob_xine_cursor_timer_id != -1)
		{
		gtk_timeout_remove(gqoob_xine_cursor_timer_id);
		gqoob_xine_cursor_timer_id = -1;
		}

	if (gqoob_xine_cursor_window && GTK_WIDGET_VISIBLE(gqoob_xine_cursor_window))
		{
		gtk_widget_hide(gqoob_xine_cursor_window);
		}
}

static void gqoob_xine_cursor_reset(void)
{
	if (gqoob_xine_cursor_timer_id != -1)
		{
		gtk_timeout_remove(gqoob_xine_cursor_timer_id);
		gqoob_xine_cursor_timer_id = -1;
		}

	gqoob_xine_cursor_timer_id = gtk_timeout_add(FULL_SCREEN_HIDE_MOUSE_DELAY, gqoob_xine_cursor_cb, NULL);
}

static void gqoob_xine_mouse_cb(GtkWidget *widget, gpointer data)
{
	if (!gqoob_xine_vw) return;

	if (gtk_xine_is_fullscreen(GTK_XINE(gqoob_xine_vw)))
		{
		gint x, y;
		GdkModifierType modmask;

		if (!gtk_xine_get_show_cursor(GTK_XINE(gqoob_xine_vw)))
			{
			gtk_xine_set_show_cursor(GTK_XINE(gqoob_xine_vw), TRUE);
			}
		gqoob_xine_cursor_reset();
		if (!gqoob_xine_cursor_window)
			{
			gqoob_xine_cursor_window = gtk_window_new(GTK_WINDOW_POPUP);
			gtk_window_set_default_size(GTK_WINDOW(gqoob_xine_cursor_window), 3, 3);
			gtk_widget_set_events(gqoob_xine_cursor_window, GDK_BUTTON_PRESS_MASK);
			gtk_signal_connect(GTK_OBJECT(gqoob_xine_cursor_window), "button_press_event",
					   GTK_SIGNAL_FUNC(gqoob_xine_mouse_press_cb), NULL);
			gtk_signal_connect(GTK_OBJECT(gqoob_xine_cursor_window), "key_press_event",
					   GTK_SIGNAL_FUNC(key_pressed), main_window);
			}
		gdk_window_get_pointer(NULL, &x, &y, &modmask);
		if (!GTK_WIDGET_VISIBLE(gqoob_xine_cursor_window)) gtk_widget_show(gqoob_xine_cursor_window);
		gtk_window_move(GTK_WINDOW(gqoob_xine_cursor_window), x - 1, y - 1);
		}
}

void gqoob_xine_fullscreen_set(gint enable)
{
	if (!gqoob_xine_vw) return;

	if (gtk_xine_is_fullscreen(GTK_XINE(gqoob_xine_vw)) == enable) return;

	gtk_xine_set_fullscreen(GTK_XINE(gqoob_xine_vw), enable, "GQoob");

	if (enable)
		{
		gqoob_xine_cursor_reset();
		}
	else
		{
		if (!gtk_xine_get_show_cursor(GTK_XINE(gqoob_xine_vw)))
			{
			gtk_xine_set_show_cursor(GTK_XINE(gqoob_xine_vw), TRUE);
			}
		gqoob_xine_cursor_disable();
		}
}

gint gqoob_xine_speed_get(void)
{
	if (!gqoob_xine_vw || !GTK_WIDGET_REALIZED(gqoob_xine_vw)) return SPEED_NORMAL;
	if (!gtk_xine_is_playing(GTK_XINE(gqoob_xine_vw)))
		{
		if (status == STATUS_PAUSE) return SPEED_PAUSE;
		return SPEED_NORMAL;
		}

	return gtk_xine_get_speed(GTK_XINE(gqoob_xine_vw));
}

void gqoob_xine_speed_set(gint speed)
{
	if (!gqoob_xine_vw || !GTK_WIDGET_REALIZED(gqoob_xine_vw)) return;
	if (status != STATUS_PLAY && status != STATUS_PAUSE) return;

	gtk_xine_set_speed(GTK_XINE(gqoob_xine_vw), speed);

	if (speed == SPEED_PAUSE)
		{
		if (status == STATUS_PLAY) status = STATUS_PAUSE;
		}
	else
		{
		if (status == STATUS_PAUSE) status = STATUS_PLAY;
		}
}

gint gqoob_xine_speed_step(gint faster)
{
	gint speed;
	
	if (!gqoob_xine_vw || !GTK_WIDGET_REALIZED(gqoob_xine_vw)) return 0;
	if (!gtk_xine_is_playing(GTK_XINE(gqoob_xine_vw)))
		{
		if (status == STATUS_PAUSE) return SPEED_PAUSE;
		return SPEED_NORMAL;
		}

	speed = gtk_xine_get_speed(GTK_XINE(gqoob_xine_vw));
	if (faster)
		{
		switch (speed)
			{
			case SPEED_PAUSE:
				speed = SPEED_SLOW_4;
				break;
			case SPEED_SLOW_4:
				speed = SPEED_SLOW_2;
				break;
			case SPEED_SLOW_2:
			default:
				speed = SPEED_NORMAL;
				break;
			case SPEED_NORMAL:
				speed = SPEED_FAST_2;
				break;
			case SPEED_FAST_2:
				speed = SPEED_FAST_4;
				break;
			case SPEED_FAST_4:
				/* already top speed */
				break;
			}
		}
	else
		{
		switch (speed)
			{
			case SPEED_FAST_4:
				speed = SPEED_FAST_2;
				break;
			case SPEED_FAST_2:
			default:
				speed = SPEED_NORMAL;
				break;
			case SPEED_NORMAL:
				speed = SPEED_SLOW_2;
				break;
			case SPEED_SLOW_2:
				speed = SPEED_SLOW_4;
				break;
			case SPEED_SLOW_4:
				speed = SPEED_PAUSE;
				break;
			case SPEED_PAUSE:
				/* already bottom speed */
				break;
			}
		}
	gqoob_xine_speed_set(speed);

	return speed;
}

gint gqoob_xine_speed_simple(gint speed)
{
	if (status == STATUS_STOP) return 0;
	if (status == STATUS_NEXT) return 4;

	switch (speed)
		{
		case SPEED_PAUSE:
			return 1;
			break;
		case SPEED_SLOW_4:
			return 2;
			break;
		case SPEED_SLOW_2:
			return 3;
			break;
		case SPEED_NORMAL:
			return 4;
			break;
		case SPEED_FAST_2:
			return 5;
			break;
		case SPEED_FAST_4:
			return 6;
			break;
		}

	return 4;
}

/*
 *----------------------------------------------------------------------------
 * more ?
 *----------------------------------------------------------------------------
 */


static SongData *gqoob_xine_current_sd = NULL;


static void gqoob_xine_data_init(SongData *sd)
{
	const gchar *ext;

	ext = extension_from_path(sd->path);

	if (ext && strcasecmp(ext, ".mp3") == 0)
		{
		mpg123_data_init(sd);
		return;
		}
	if (ext && strcasecmp(ext, ".ogg") == 0)
		{
		ogg123_data_init(sd);
		return;
		}

	sd->type_description = _("Generic media");
	sd->live = FALSE;
}

static gint gqoob_xine_data_set(SongData *sd, gint read_info)
{
	if (read_info && !sd->info_loaded)
		{
		const gchar *ext;

		ext = extension_from_path(sd->path);

		/* handle extended types (files we know how to obtain info from) */
		if (ext && strcasecmp(ext, ".mp3") == 0) return mpg123_data_set(sd, read_info);
		if (ext && strcasecmp(ext, ".ogg") == 0) return ogg123_data_set(sd, read_info);

		sd->info_loaded = TRUE;
		}

	return TRUE;
}

static GtkWidget *gqoob_xine_info_window(const gchar *file)
{
	const gchar *ext;

	ext = extension_from_path(file);
	if (!ext) return NULL;
	
	/* mp3 */
	if (strcasecmp(ext, ".mp3") == 0) return mpg_create_info_window(file);

	/* ogg */
	if (strcasecmp(ext, ".ogg") == 0) return ogg123_info_window(file);

	return NULL;
}

/*
 *----------------------------------------------------------------------------
 * timer update funcs
 *----------------------------------------------------------------------------
 */

static gint gqoob_xine_timer_id = -1;

static gint gqoob_xiner_timer_cb(gpointer data)
{
	if (gqoob_xine_timer_id == -1) return FALSE;

	if (gqoob_xine_vw && gqoob_xine_current_sd)
		{
		gint length;
		
		length = gtk_xine_get_total_time(GTK_XINE(gqoob_xine_vw));
		seconds = gtk_xine_get_current_time(GTK_XINE(gqoob_xine_vw));
		seconds_remaining = length - seconds;
		}
	
	return TRUE;
}

static void gqoob_xine_timer(gint enable)
{
	if (enable && gqoob_xine_timer_id == -1)
		{
		gqoob_xine_timer_id = gtk_timeout_add(1000, gqoob_xiner_timer_cb, NULL);
		}
	else if (!enable && gqoob_xine_timer_id != -1)
		{
		gtk_timeout_remove(gqoob_xine_timer_id);
		gqoob_xine_timer_id = -1;
		}
}

/*
 *----------------------------------------------------------------------------
 * module play funcs
 *----------------------------------------------------------------------------
 */

static gint gqoob_xine_start(SongData *sd, gint position)
{
	gint ret = FALSE;

	gqoob_xine_current_sd = sd;

	if (gqoob_xine_vw)
		{
		ret = gtk_xine_play(GTK_XINE(gqoob_xine_vw), sd->path, 0, position);
		gqoob_xine_timer(TRUE);
		}

	playback_done_command(EXEC_PLAY, !ret);
	display_xine_speed_update();
	return ret;
}

static gint gqoob_xine_stop(SongData *sd)
{
	gint ret = FALSE;

	if (gqoob_xine_vw)
		{
		if (gtk_xine_is_playing(GTK_XINE(gqoob_xine_vw)))
			{
			gtk_xine_stop(GTK_XINE(gqoob_xine_vw));
			}
		ret = TRUE;
		}

	gqoob_xine_timer(FALSE);

	gqoob_xine_current_sd = NULL;

	playback_done_command(EXEC_STOP, !ret);
	return ret;
}

static gint gqoob_xine_pause(SongData *sd)
{
	gint ret = FALSE;

	if (gqoob_xine_vw)
		{
		if (gqoob_xine_current_sd != sd)
			{
			printf("warning: songdata inconsistency for pause\n");
			}
		gtk_xine_set_speed(GTK_XINE(gqoob_xine_vw), SPEED_PAUSE);
		ret = TRUE;
		}

	playback_done_command(EXEC_PAUSE, !ret);
        return ret;
}

static gint gqoob_xine_continue(SongData *sd)
{
	gint ret = FALSE;

	if (gqoob_xine_vw)
		{
		if (gqoob_xine_current_sd != sd)
			{
			printf("warning: songdata inconsistency for continue\n");
			}
		gtk_xine_set_speed(GTK_XINE(gqoob_xine_vw), SPEED_NORMAL);
		ret = TRUE;
		}

	playback_done_command(EXEC_CONTINUE, !ret);
	return ret;
}

static gint gqoob_xine_seek(SongData *sd, gint position)
{
	gint ret = FALSE;

	if ((gqoob_xine_current_sd && gqoob_xine_current_sd->live) ||
	    (gqoob_xine_vw && !gtk_xine_is_seekable(GTK_XINE(gqoob_xine_vw))))
		{
		/* not seekable */
		ret = TRUE;
		}
	else if (gqoob_xine_vw && gqoob_xine_current_sd)
		{
		gint speed;

		if (gqoob_xine_current_sd != sd)
			{
			printf("warning: songdata inconsistency for seek\n");
			}

		speed = gtk_xine_get_speed(GTK_XINE(gqoob_xine_vw));
		gtk_xine_play(GTK_XINE(gqoob_xine_vw), gqoob_xine_current_sd->path, 0, position);
		if (speed != SPEED_NORMAL) gtk_xine_set_speed(GTK_XINE(gqoob_xine_vw), speed);

		/* let display be a little more accurate upon slide release */
		seconds_remaining += seconds - position;
		seconds = position;

		ret = TRUE;
		}

	playback_done_command(EXEC_SEEK, !ret);

	return ret;
}

static void gqoob_xine_end_of_stream_cb(GtkWidget *widget, gpointer data)
{
	if (gqoob_xine_current_sd != current_song_get_data())
		{
		printf("warning: songdata inconsistency for eos\n");
		}
	gqoob_xine_timer(FALSE);
	if (status == STATUS_PLAY) module_playback_end();
	gqoob_xine_current_sd = NULL;
}

static void gqoob_xine_error_cb(GtkWidget *widget, GtkXineError error, const char *message, gpointer data)
{
	if (error == 0 || error == GTX_STARTUP) return;
	gqoob_xine_timer(FALSE);
	module_playback_error(gqoob_xine_current_sd);
	gqoob_xine_current_sd = NULL;
}

/*
 *----------------------------------------------------------------------------
 * player module interface routines
 *----------------------------------------------------------------------------
 */

void gqoob_xine_finish(void)
{
	GtkWidget *fixed;
	const gchar *video;
	const gchar *audio;
	gchar *logo_buf;

	printf("io_xine.c: finishing up at %d, %c (%d x %d)\n", gqoob_xine_x, gqoob_xine_y, gqoob_xine_w, gqoob_xine_h);

	gtk_widget_ref(main_window->display);
	fixed = gtk_fixed_new();
	gqoob_xine_fixed = fixed;
	
	gtk_widget_reparent(main_window->display, fixed);
	gtk_container_add(GTK_CONTAINER(main_window->window), fixed);
	gtk_widget_show(fixed);
	gtk_fixed_move(GTK_FIXED(fixed), main_window->display, 0, 0);

	if (gqoob_xine_override_video)
		{
		printf(_("Forcing video plugin \"%s\"\n"), gqoob_xine_override_video);
		g_free(gqoob_xine_output_video);
		gqoob_xine_output_video = gqoob_xine_override_video;
		gqoob_xine_override_video = NULL;
		}

	if (gqoob_xine_output_video && strcmp(gqoob_xine_output_video, "auto") == 0)
		{
		video = NULL;
		}
	else
		{
		video = gqoob_xine_output_video;
		}

	if (gqoob_xine_override_audio)
		{
		printf(_("Forcing audio plugin \"%s\"\n"), gqoob_xine_override_audio);
		g_free(gqoob_xine_output_audio);
		gqoob_xine_output_audio = gqoob_xine_override_audio;
		gqoob_xine_override_audio = NULL;
		}

	if (gqoob_xine_output_audio && strcmp(gqoob_xine_output_audio, "auto") == 0)
		{
		audio = NULL;
		}
	else
		{
		audio = gqoob_xine_output_audio;
		}

	gqoob_xine_vw = gtk_xine_new_with_plugins(gqoob_xine_w, gqoob_xine_h, video, audio);
	logo_buf = g_strconcat(GQOOB_SKINDIR, "/gqoob-logo.mpv", NULL);
	if (isfile(logo_buf))
		{
		gtk_xine_set_logo(GTK_XINE(gqoob_xine_vw), logo_buf);
		}
	g_free(logo_buf);
	gtk_signal_connect(GTK_OBJECT(gqoob_xine_vw), "eos",
			   GTK_SIGNAL_FUNC(gqoob_xine_end_of_stream_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(gqoob_xine_vw), "error",
			   GTK_SIGNAL_FUNC(gqoob_xine_error_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(gqoob_xine_vw), "mouse_motion",
			   GTK_SIGNAL_FUNC(gqoob_xine_mouse_cb), NULL);

	gtk_widget_set_size_request(gqoob_xine_vw, gqoob_xine_w, gqoob_xine_h);
	gtk_fixed_put(GTK_FIXED(fixed), gqoob_xine_vw, gqoob_xine_x, gqoob_xine_y);
	gtk_widget_show(gqoob_xine_vw);

	gtk_widget_unref(main_window->display);
	gtk_widget_show(main_window->display);
}

void gqoob_xine_ready(void)
{
	if (!gqoob_xine_vw) return;

	/* allow realizations, etc. */
	while (gtk_events_pending()) gtk_main_iteration();
	
	/* well, aparently this should be done to make sure xine is ready to go... */
	while (gtk_xine_check(GTK_XINE(gqoob_xine_vw)) == FALSE) usleep(100000);
}

static gint gqoob_xine_move_cb(gpointer data)
{
	GtkAllocation a;

	if (gqoob_xine_idle_id == -1) return FALSE;

	a.x = gqoob_xine_x;
	a.y = gqoob_xine_y;
	a.width = gqoob_xine_w;
	a.height = gqoob_xine_h;
	gtk_widget_size_allocate(gqoob_xine_vw, &a);

	gqoob_xine_idle_id = -1;
	return FALSE;
}

void gqoob_xine_move(gint x, gint y, gint width, gint height, gint hidden)
{
	if (hidden)
		{
		/* just move it out of view when hidden */
		gqoob_xine_x = 4096;
		gqoob_xine_y = 4096;
		gqoob_xine_w = 32;
		gqoob_xine_h = 32;
		}
	else
		{
		gqoob_xine_x = x;
		gqoob_xine_y = y;
		gqoob_xine_w = width;
		gqoob_xine_h = height;
		}

	if (!gqoob_xine_fixed || !gqoob_xine_vw) return;

	if (gqoob_xine_idle_id == -1) gqoob_xine_idle_id = gtk_idle_add(gqoob_xine_move_cb, NULL);
}

void gqoob_xine_init(void)
{
	IO_ModuleData *imd;
	gint id;

	if (debug_mode) printf("XINE module initing...\n");

	imd = g_new0(IO_ModuleData, 1);

	imd->title = "xine";
	imd->description = _("XINE media playback library");

	imd->songdata_init_func = gqoob_xine_data_init;
	imd->songdata_info_func = gqoob_xine_data_set;

	imd->start_func = gqoob_xine_start;
	imd->stop_func = gqoob_xine_stop;
	imd->pause_func = gqoob_xine_pause;
	imd->continue_func = gqoob_xine_continue;
	imd->seek_func = gqoob_xine_seek;

	imd->info_func = gqoob_xine_info_window;

	imd->config_load_func = gqoob_xine_config_load;
	imd->config_save_func = gqoob_xine_config_save;
	imd->config_init_func = gqoob_xine_config_init;
	imd->config_apply_func = gqoob_xine_config_apply;
	imd->config_close_func = gqoob_xine_config_close;

	id = player_module_register(imd);

	gqoob_xine_register_types(imd, id);

	ogg123_init();
}
