/*
 * GQoob
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqoob.h"
#include "io_xine.h"

#include "io_mpg123.h"
#include "io_ogg123.h"
#include "rcfile.h"

static GtkWidget *config_vbox = NULL;
static GtkWidget *config_vmenu = NULL;
static GtkWidget *config_amenu = NULL;


static const gchar *output_menu_get(GtkWidget *omenu, GList *list);

static gint c_mpg123_read_id3_tags = TRUE;
static gint c_ogg123_read_info = TRUE;


/*
 *----------------------------------------------------------------------------
 * load / save routines
 *----------------------------------------------------------------------------
 */

void gqoob_xine_config_load(FILE *f, const gchar *option, const gchar *value, const gchar *value_all)
{
	gqoob_xine_debug = read_bool_option(f, option,
		"xine_output_debug", value, gqoob_xine_debug);
	gqoob_xine_output_video = read_char_option(f, option,
		"xine_output_video", value, gqoob_xine_output_video);
	gqoob_xine_output_audio = read_char_option(f, option,
		"xine_output_audio", value, gqoob_xine_output_audio);

	mpg123_read_id3_tags = read_bool_option(f, option,
		"read_id3_tags", value, mpg123_read_id3_tags);
	ogg123_read_info = read_bool_option(f, option,
		"read_ogg_info", value, ogg123_read_info);
}

void gqoob_xine_config_save(FILE *f)
{
	fprintf(f,"\n##### Xine Options #####\n\n");
	
	write_bool_option(f, "xine_output_debug", gqoob_xine_debug);
	write_char_option(f, "xine_output_video", gqoob_xine_output_video);
	write_char_option(f, "xine_output_audio", gqoob_xine_output_audio);

	write_bool_option(f, "read_id3_tags", mpg123_read_id3_tags);
	write_bool_option(f, "read_ogg_info", ogg123_read_info);
}

/*
 *----------------------------------------------------------------------------
 * config window routines
 *----------------------------------------------------------------------------
 */

static void checkbox_cb(GtkWidget *w, gpointer data)
{
	gint *c_n = data;

	*c_n = GTK_TOGGLE_BUTTON (w)->active;
}

static GtkWidget *checkbox_new(gchar *text, gint n, gint *c_n)
{
	GtkWidget *button;

	*c_n = n;
	button = gtk_check_button_new_with_label(text);
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), n);
	gtk_signal_connect (GTK_OBJECT(button), "clicked", (GtkSignalFunc)checkbox_cb, c_n);

	return button;
}

void gqoob_xine_config_apply(void)
{
	if (!config_vbox) return;

	g_free(gqoob_xine_output_video);
	gqoob_xine_output_video = g_strdup(output_menu_get(config_vmenu, gqoob_xine_get_video_list()));
	
	g_free(gqoob_xine_output_audio);
	gqoob_xine_output_audio = g_strdup(output_menu_get(config_amenu, gqoob_xine_get_audio_list()));

	mpg123_read_id3_tags = c_mpg123_read_id3_tags;
	ogg123_read_info = c_ogg123_read_info;
}

void gqoob_xine_config_close(void)
{
	config_vbox = NULL;
}

static const gchar *output_menu_get(GtkWidget *omenu, GList *list)
{
	gint n;

	n = gtk_option_menu_get_history(GTK_OPTION_MENU(omenu));

	return g_list_nth_data(list, n);
}

static GtkWidget *output_menu_new(GList *list, const gchar *current)
{
	GtkWidget *omenu;
	GtkWidget *menu;
	GList *work;
	gint c;
	gint index = 0;

	omenu = gtk_option_menu_new();
	menu = gtk_menu_new();

	work = list;
	c = 0;
	while (work)
		{
		gchar *handle = work->data;
		GtkWidget *item;

		if (current && strcmp(current, handle) == 0) index = c;

		item = gtk_menu_item_new_with_label(handle);
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_widget_show(item);
		c++;
		work = work->next;
		}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), index);

        return omenu;
}

GtkWidget *gqoob_xine_config_init(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *frame;
	GtkWidget *button;
	GtkWidget *label;

	config_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(config_vbox), 5);

	frame = gtk_frame_new(_("Output options"));
	gtk_container_border_width(GTK_CONTAINER(frame), 5);
	gtk_box_pack_start(GTK_BOX(config_vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show(vbox);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Video plugin:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	config_vmenu = output_menu_new(gqoob_xine_get_video_list(), gqoob_xine_output_video);
        gtk_box_pack_start(GTK_BOX(hbox), config_vmenu, FALSE, FALSE, 0);
        gtk_widget_show(config_vmenu);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Audio plugin:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	config_amenu = output_menu_new(gqoob_xine_get_audio_list(), gqoob_xine_output_audio);
	gtk_box_pack_start(GTK_BOX(hbox), config_amenu, FALSE, FALSE, 0);
	gtk_widget_show(config_amenu);

	label = gtk_label_new(_("Plugin changes require restart."));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	button = checkbox_new(_("Use ID3 tags when available"),
		 mpg123_read_id3_tags, &c_mpg123_read_id3_tags);
	gtk_box_pack_start(GTK_BOX(config_vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = checkbox_new(_("Read ogg length and comments"),
		 ogg123_read_info, &c_ogg123_read_info);
	gtk_box_pack_start(GTK_BOX(config_vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

#if 0
	button = checkbox_new(_("Debugging output to console"), gqoob_xine_debug, &gqoob_xine_debug);
	gtk_box_pack_start(GTK_BOX(config_vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);
#endif

	return config_vbox;
}

